/*--------------------------------------------------------------------------+
$Id: CLASS.java 11213 2016-03-04 16:27:08Z MONDAL $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.builder;

import static java.util.Objects.requireNonNull;
import static org.fortiss.tooling.spiderchart.gc.AbstractChartColor.WHITE;
import static org.fortiss.tooling.spiderchart.gc.AbstractChartColor.YELLOW;
import static org.fortiss.tooling.spiderchart.gc.AbstractGraphicsSupplier.getColor;
import static org.fortiss.tooling.spiderchart.style.FillStyle.GRADIENT_VERTICAL;

import java.util.function.Consumer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.fortiss.tooling.spiderchart.SpiderChart;
import org.fortiss.tooling.spiderchart.style.FillStyle;
import org.fortiss.tooling.spiderchart.swt.SpiderChartViewer;

/**
 * Used to build the complete spider chart
 *
 * @author AMIT KUMAR MONDAL
 *
 */
public final class SpiderChartBuilder {

	/** Spider Chart Configuration Builder Ref */
	private static SpiderChartConfigurationBuilder chartConfiguration;

	/** Chart Viewer Builder */
	private static SpiderChartBuilder chartViewerBuilder;

	/** Configuration for the Spider Chart */
	public static SpiderChartBuilder config(final Composite parent,
			final Consumer<SpiderChartConfigurationBuilder> settings) {
		requireNonNull(parent);
		requireNonNull(settings);
		chartConfiguration = new SpiderChartConfigurationBuilder();
		settings.accept(chartConfiguration);
		chartViewerBuilder = new SpiderChartBuilder(parent);
		return chartViewerBuilder;
	}

	/** Actual Spider Chart to be used */
	private SpiderChart chart;

	/** Actual Spider Chart Viewer */
	private final SpiderChartViewer chartViewer;

	/** Constructor */
	public SpiderChartBuilder(final Composite parent) {
		requireNonNull(parent);
		this.chartViewer = new SpiderChartViewer(parent, SWT.NONE);
		this.prepareChartViewer(parent);
	}

	/** The datapoints provider */
	public SpiderChartBuilder data(final Consumer<AxisDataBuilder> dataBuilderConsumer) {
		requireNonNull(dataBuilderConsumer);
		final AxisDataBuilder dataBuilder = new AxisDataBuilder(this.chart);
		dataBuilderConsumer.accept(dataBuilder);
		this.chartViewer.setChart(this.chart);
		return this;
	}

	/** Configuration for the spider chart */
	private void prepareChartViewer(final Composite parent) {
		this.chartViewer.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		this.chartViewer.setBounds(parent.getShell().getClientArea().x, parent.getShell()
				.getClientArea().y, parent.getShell().getClientArea().width, parent.getShell()
				.getClientArea().height - 10);

		// Some chart related default configurations
		this.chart =
				new SpiderChart(chartConfiguration.getTitle(), chartConfiguration.getPlotter());
		this.chart.setBackStyle(new FillStyle(getColor(YELLOW)));
		this.chart.setBackgroundCanvasColor(WHITE);
		this.chart.getBackStyle().setGradientType(GRADIENT_VERTICAL);
		this.chart.setLegend(chartConfiguration.getLegend());
		this.chart.setRepaintAll(true);
		this.chart.setActivateSelection(true);
	}

	/** Builds the viewer */
	public SpiderChartViewer viewer(final Consumer<SpiderChartBuilder> chartBuilderConsumer) {
		requireNonNull(chartBuilderConsumer);
		chartBuilderConsumer.accept(chartViewerBuilder);
		return chartViewerBuilder.chartViewer;
	}

	/** Builds the viewer if values are already provided to the plotter using line data sequence */
	public SpiderChartViewer viewer() {
		this.chartViewer.setChart(this.chart);
		return chartViewerBuilder.chartViewer;
	}

}
